@@ -24,10 +24,13 @@ def get_gtk_keymap(ignore_keys=[None, "VoidSymbol"], add_if_missing=[]):
         by adding the keyval_name.
         We can also ignore some keys
     """
-    import pygtk
-    pygtk.require("2.0")
-    import gtk
-    keymap = gtk.gdk.keymap_get_default()
+    from wimpiggy.gobject_compat import import_gdk
+    gdk = import_gdk()
+    try:
+        keymap = gdk.keymap_get_default()
+    except:
+        keymap = None
+        return  []
     keycodes=[]
     max_entries = 1
     for i in range(0, 2**8):
@@ -35,7 +38,7 @@ def get_gtk_keymap(ignore_keys=[None, "VoidSymbol"], add_if_missing=[]):
         if entries:
             max_entries = max(max_entries, len(entries))
             for keyval, keycode, group, level in entries:
-                name = gtk.gdk.keyval_name(keyval)
+                name = gdk.keyval_name(keyval)
                 if name not in ignore_keys:
                     keycodes.append((nn(keyval), nn(name), nn(keycode), nn(group), nn(level)))
                 if name in add_if_missing:
@@ -47,6 +50,14 @@ def get_gtk_keymap(ignore_keys=[None, "VoidSymbol"], add_if_missing=[]):
     return keycodes
 
 
+def main():
+    gtk_keymap = get_gtk_keymap()
+    print("gtk_keymap: (keyval, name, keycode, group, level)\n%s" % ("\n".join([str(x) for x in gtk_keymap])))
+
+if __name__ == "__main__":
+    main()
+
+
 
 DEFAULT_MODIFIER_IGNORE_KEYNAMES = ["Caps_Lock", "Num_Lock", "Scroll_Lock"]
 
@@ -65,33 +76,6 @@ DEFAULT_MODIFIER_NAMES = ["shift", "control", "meta", "super", "hyper", "alt"]
 DEFAULT_MODIFIER_NUISANCE = ["lock", "num", "scroll"]
 DEFAULT_ALL_MODIFIER_NAMES = DEFAULT_MODIFIER_NAMES+DEFAULT_MODIFIER_NUISANCE+["mod1", "mod2", "mod3", "mod4", "mod5"]
 
-XMODMAP_MOD_CLEAR = ["clear Lock", "clear Shift", "clear Control",
-                 "clear Mod1", "clear Mod2", "clear Mod3", "clear Mod4", "clear Mod5"]
-XMODMAP_MOD_ADD = ["add Lock = Caps_Lock",
-                 "add Shift = Shift_L Shift_R",
-                 "add Control = Control_L Control_R",
-                 "add Mod1 = Meta_L Meta_R",
-                 "add Mod2 = Num_Lock",
-                 "add Mod3 = Hyper_L Hyper_R",
-                 "add Mod4 = Super_L Super_R",
-                 "add Mod5 = Alt_L Alt_R"]
-
-XMODMAP_MOD_DEFAULTS = ["keycode any = Shift_L",
-                   "keycode any = Shift_R",
-                   "keycode any = Control_L",
-                   "keycode any = Control_R",
-                   "keycode any = Meta_L",
-                   "keycode any = Meta_R",
-                   "keycode any = Alt_L",
-                   "keycode any = Alt_R",
-                   "keycode any = Hyper_L",
-                   "keycode any = Hyper_R",
-                   "keycode any = Super_L",
-                   "keycode any = Super_R",
-                   "keycode any = Num_Lock",
-                    # Really stupid hack to force backspace to work.
-                   "keycode any = BackSpace"]
-
 DEFAULT_MODIFIER_MEANINGS = {
         "Shift_L"   : "shift",
         "Shift_R"   : "shift",
@@ -110,14 +94,3 @@ DEFAULT_MODIFIER_MEANINGS = {
         "ISO_Level3_Shift"  : "mod5",
         "Mode_switch"       : "mod5",
         }
-
-DEFAULT_KEYNAME_FOR_MOD = {
-            "shift": ["Shift_L", "Shift_R"],
-            "control": ["Control_L", "Control_R"],
-            "meta": ["Meta_L", "Meta_R"],
-            "super": ["Super_L", "Super_R"],
-            "hyper": ["Hyper_L", "Hyper_R"],
-            "alt": ["Alt_L", "Alt_R"],
-            "lock": ["Caps_Lock"],
-            "num": ["Num_Lock"],
-            }
